# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class ExecutorsRepo(object):
    def __init__(self):
        self._executors = {}

    def register_executor(self, name, cls):
        self._executors[name] = cls

    def create_executor(self, type, name):
        if type not in self._executors:
            return None
        return self._executors[type](name)


Executors = ExecutorsRepo()


def executor(name):
    def wrapper(cls):
        Executors.register_executor(name, cls)
        return cls
    return wrapper
